<?php
/**
 * Contains Classes for the Favicon Content Replacers.
 *
 * This file contains all the clases for actually replacing the
 * favicons in the content and the widgets.
 *
 * @package WP-Favicons
 * @since 0.1
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */
namespace leau\co\wp_favicons;
/**
 * Class: EdlLinkFaviconContent
 *
 * This class adds a favicon to a link based on the generic class EdlHtmlHref and in the first place based on the public google favicons
 * though one can add a physical icon in the file cache so that this one will take preference.
 *
 * @package WP-Favicons
 * @since 0.1
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */

ini_set('gd.jpeg_ignore_warning', 1);

// note that fully quantified namespaces should be used
// when referencing a class from another scope
if (!class_exists("EdlLinkFaviconContent") && class_exists("leau\co\wp_favicons\EdlHtmlHref"))
{
	class EdlLinkFaviconContent extends EdlHtmlHref
	{
		private $url;
		public  $_filter = 'the_content';

		/**
		 * Method: constructor
		 *
		 *
		 *
		 * @param enum $replaceContext (optional) determines if the context is widgets or main content
		 * @since 0.1
		 */
		public function __construct($edllinkFaviconContext)
		{
			$this->_filter = $edllinkFaviconContext;
			add_filter($this->_filter, array($this,'ReplaceAll'), 9);

			// @todo 0.5.0 make this more generic for filetypes plugins
			$admin_options = Config::GetOptionsAsArray();
			if (isset($admin_options[Config::GetPluginSlug() . 'exclude_filetypes'])) {
				parent::SetNotAllowedExtension($admin_options[Config::GetPluginSlug() . 'exclude_filetypes']);
			}
		}

		/**
		 * Traverses all active plugins for a favicon
		 * Plugins can hook in the filters in this method, only one of them can win
		 * as soon as one has found an icon the loop stops. So only one icon is
		 * returned in the final resulting array.
		 * @param $url
		 * @return array $faviconarray
		 */
		public function getIcon($url)
		{
			/* Start off with a check that if the URI exists in "the" database
			 *
			 */
			Database::DbUrlCheck($url);

			$favicon_array = array();
			/*
			 * Filter: Favicon Search
			 *
			 * You can hook into this filter to add your own favicon search engines or extractors e.g. :
			 * - you can use public favicon search engines and provide the results,
			 * - you can even store all links in a database and do totally other stuff e.g. list all links in a blog with amount of views
			 *
			 * Your filter should return the $mArrfavicon_list array.
			 *
			 * - the first array element is what you need to use always to indicate the type e.g.: 'png', 'gif', etc...
			 * - the second array element holds the RAW DATA of the image, so that we can write it to the cache
			 * - the id of the image in the database
			 *
			 * If you receive an error do NOT return the array since we do not want rubbish in the array an at this stage
			 * do the error handling in the do_filter function it self.
			 *
			 * you can change the existing data (e.g. refine the image with image tooling) or you can add extra favicons
			 * the favicon shown on the resulting page is simply done by alphabetical order
			 * see: http://core.trac.wordpress.org/ticket/901 for a nice thread
			 *
			 * EXAMPLES: see the /filters subdirectory
			 *
			 */
			$favicon_array = apply_filters( Config::GetPluginSlug() . 'search', /*array*/ $favicon_array, /*string*/ $url );

			/*
			 * Filter: Defaults which are saved to disk
			 *
			 * The default filters provide default icons
			 * They work exactly the same as the search filters but a) to make sure they
			 * are always "last in the list" (although this could be controlled by the
			 * priority) and b) to make it more obvious, a seperate filter is provided.
			 */
			$favicon_array = apply_filters( Config::GetPluginSlug() . 'default', /*array*/ $favicon_array, /*string*/ $url );


			/*
			 * Filter: Conversion
			 *
			 * We will convert all images to png, see: http://en.wikipedia.org/wiki/Portable_Network_Graphics
			 *    although .ico: http://en.wikipedia.org/wiki/Favicon is better supported over all browsers
			 * The reason for this is that originally .ico was the only way to have an icon appear but later more
			 * filetypes are supported. If a website uses a later filetype png produces better results than ico.
			 * The disadvantage is that browsing the cache directory under e.g. Windows XP with .ico files produces much
			 * a much nicer browsing experience since the .ico are shown as uhm... icons.
			 *
			 * However... if you dont like PNG but would rather have GIF, ICO, whatever then just write your own filter,
			 * the only thing to remember is to set the correct cache_file_name
			 *
			 * You can also write manipulators e.g. give all icons a dark corner if you want to.
			 *
			 * We provide some default conversion filters but you add your own using the filter to convert an image.
			 *
			 */
			$favicon_array = apply_filters( Config::GetPluginSlug() . 'convert', /*array*/ $favicon_array );

			/*
			 * Filter: Defaults are not saved to disk
			 * (also failsafe when filters fail)
			 * The default filters provide default icons
			 * They work exactly the same as the search filters but a) to make sure they
			 * are always "last in the list" (although this could be controlled by the
			 * priority) and b) to make it more obvious, a seperate filter is provided.
			 * @todo 0.5.0 make 2 types of defaults: cached or non cached
			 */

			$favicon_array = apply_filters( Config::GetPluginSlug() . 'default', /*array*/ $favicon_array, /*string*/ $url );

			/* now select 1 icon */

			return $favicon_array;
		}

		/*
		 * regular cache modus image check
		 */
		function ImageCheck($url)
		{
			/*
			 * if not in database then assume NEW item
			 * and run the favicon loop to get it in Db
			 *
			 * if it is not in the physical cache we need
			 * to run it anway
			 */

			$imageinfo = Database::GetIconInfo($url);
			$exists_in_database = true;
			if (!$imageinfo || (false == $imageinfo))
			{
				$exists_in_database = false;
			}

			/* if not in database... get the data
			 * and put it in the database
			 */
			$favicon_array = array();
			if (false == $exists_in_database)
			{
				$favicon_array = $this->getIcon($url);
				$imageinfo = Database::GetIconInfo($url);
			}
			else
			{
				/* even if it is in the database the information
				 * could be outdated e.g. user saved previously icons as .ico
				 * and has now turned on a filter to save to .png OR user
				 * wanted icons of google only before and has now switched on
				 * page grabbing.
				 * We rsolve this functional and ask the user to clean the cache
				 * after changing options.
				 */

			}

			/*
			 * if still not in database... then the user has disabled
			 * defaults but is still using the cache in this case return
			 * nothing
			 */

			if (false == $imageinfo)
			{
				return "";
			}

			/* configure cache based on Db setting (filetype to be found)
			 * now we have an activated cache
			 */
			$favicon_cache = new EdlLinkFaviconCache(
				$url, 'favicon.' . $imageinfo->favicon);

			/*
			 * a) if was not in database then save a version in the cache
			 * b) if the user deleted the item himself in the cache
			 */

			if (false == $exists_in_database)
			{
				$data='';
				if ($favicon_array && array_key_exists('data',$favicon_array))
				{
					$data = $favicon_array['data'];
					$favicon_cache->Savedata($data);
				}
				else
				{
					/*
					 * if nothing is found and the user has supplied no
					 * defaults nothing is returned
					 *
					 */
					return "";
				}
			}

			return array($imageinfo,$favicon_cache);
		}


		/**
		 * Method: GetIconHTML
		 *
		 * Creates and returns the Icon HTML to be added after the href.
		 *
		 * @since 0.1
		 */
		function GetIconHTML($url)
		{
			$image = "";
			$use_cache = Config::GetOptionsArrayValue(Config::GetPluginSlug() .
				'use_cache');
			if (!$use_cache)
			{
				/* if no cache then dummy modus */

				$image = 'http://www.google.com/s2/favicons?domain=' .
				parse_url($url, PHP_URL_HOST);
			}
			else
			{
				/* Imagecheck
				 * makes sure the database entry is there and if new
				 * creates a cached version
				 */

				$dbCheckArray = $this->ImageCheck($url);
				if (!$dbCheckArray) {return false;}
				$imageinfo = $dbCheckArray[0];
				$favicon_cache = $dbCheckArray[1];

				if (!$favicon_cache->FileExistInCache())
				{
					// be deleting an icon in the cache manually
					// a user indicates he does not want to show
					// that icon. it could also be the case that
					// operations were stopped and icons are missing
					// but only for filter operations otherwise the
					// icon would be in the cache however the filter
				 	// operates on data from earlier finds since we
				 	// do not want to store these in the db.
					// @todo 0.5.0 followup on this use case

					return;
				}

				/*
				 * the image string should now always contain a
				 * file existing in the cache
				 */
				$image = $favicon_cache->GetRelativeLocation()
				. 'favicon.' . $imageinfo->favicon;
			}


			return '<img src="' .
			$image .
				'" alt="' . Config::GetPluginSlug() . '" class="' .
			Config::GetPluginSlug() . $this->_filter . '" />';
		}

		/**
		 * (non-PHPdoc)
		 * @see edl-classes/EdlHtmlHref::SpecificHrefBefore()
		 */
		public function SpecificHrefBefore( /*string*/ $strHtmlUri)
		{
			$this->url = $strHtmlUri;
			return $this->GetIconHtml($this->url);
		}
	}
}